from eth_keyfile import decode_keyfile_json, create_keyfile_json
from web3 import Web3

# 先置换， 再加密
def encrypt(mnemonic:str, password: str, exchange = True):
    if exchange:
        mnemonic = mnemonic.strip().split(' ')
        mnemonic[2], mnemonic[8] = mnemonic[8],mnemonic[2]
        mnemonic[7], mnemonic[15] = mnemonic[15],mnemonic[7]
        mn =  " ".join(mnemonic)
    else:
        mn = mnemonic
    return create_keyfile_json(mn.encode(), password.encode())

def decrypt(mnemonic:str, password: str, exchange = True):
    mnemonic = mnemonic.strip().split(' ')
    mnemonic[2], mnemonic[8] = mnemonic[8],mnemonic[2]
    mnemonic[7], mnemonic[15] = mnemonic[15],mnemonic[7]
    return " ".join(mnemonic)





from ether.client import *
from ether.abis import erc20
from ether.wallet import Wallet

def get_erc20_balance(client: Web3Client, token, address):
    contract = client.eth.contract(token, abi = erc20)
    return contract.functions.balanceOf(address).call()


if __name__ == '__main__':
    client = Web3Client['bsc']
    wallet = Wallet(mn)[1:100]
    for i in wallet:
        print(get_erc20_balance(client, '0x555296de6A86E72752e5C5DC091FE49713Aa145C',i.address))



from concurrent.futures import ThreadPoolExecutor
import multiprocessing
import threading
from typing import Callable
from web3 import Web3
from web3.middleware import Middleware
from web3.types import RPCEndpoint, RPCResponse
from typing import Any
from .provider import FlashbotProvider

FLASHBOTS_METHODS = [
    "eth_sendBundle",
]


def construct_flashbots_middleware(
    flashbots_providers: list[FlashbotProvider],
) -> Middleware:
    """Captures Flashbots RPC requests and sends them to the Flashbots endpoint
    while also injecting the required authorization headers

    Keyword arguments:
    flashbots_provider -- An HTTP provider instantiated with any authorization headers
    required
    """

    def flashbots_middleware(
        make_request: Callable[[RPCEndpoint, Any], Any], w3: Web3
    ) -> Callable[[RPCEndpoint, Any], RPCResponse]:
        def middleware(method: RPCEndpoint, params: Any) -> RPCResponse:
            if method not in FLASHBOTS_METHODS:
                return make_request(method, params)
            else:
                with ThreadPoolExecutor(len(flashbots_providers)) as pool:
                    result = list(
                        pool.map(
                            lambda provider: provider.make_request(method, params),
                            flashbots_providers,
                        )
                    )
                    return {"jsonrpc": "2.0", "id": 0, "result": result}

        return middleware

    return flashbots_middleware



import json
import logging
import websocket


class WsClient:
    def __init__(self, url, params) -> None:
        self.params = params
        self.url = url
        self.cli = websocket.WebSocket()

    def start(self):
        self.cli.connect(self.url)
        self.cli.send(
            json.dumps(
                {
                    "id": 1,
                    "jsonrpc": "2.0",
                    "method": "eth_subscribe",
                    "params": self.params,
                }
            )
        )
        response = json.loads(self.cli.recv())
        logging.info(f"start subscription: {self.params[0]}, {response}")
        return self

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, type, value, trace):
        self.cli.close()
        logging.info(f"stop subscription: {self.params[0]}")

    def __iter__(self):
        return map(lambda x: json.loads(x)['params']['result'], self.cli)


import secrets
from eth_account import Account
from web3 import Web3
from eth_abi import encode
from eth_abi.packed import encode_packed
# from ether.abis import erc20

FactoryAddrV3 = Web3.to_bytes(hexstr="0x1F98431c8aD98523631AE4a59f267346ea31F984")
FactoryAddrV2 = Web3.to_bytes(hexstr="0x5c69bee701ef814a2b6a3edd4b1652cb9cc5aa6f")

PoolInitCodeV3 = Web3.to_bytes(hexstr="0xe34f199b19b2b4f47f68442619d555527d244f78a3297ea89325f843f87b8b54")
PoolInitCodeV2 = Web3.to_bytes(hexstr="0x96e8ac4277198ff8b6f785478aa9a39f403cb768dd02cbee326c3e7da348845f")

# 计算v2 v3 pool 地址, 抢开盘需要
def sort_addrs(addr1: str, addr2: str) -> (str,str):
    b1 = Web3.to_bytes(hexstr=addr1)
    b2 = Web3.to_bytes(hexstr=addr2)
    return (addr1, addr2) if b1 < b2 else (addr2,addr1)


def get_v2_pool_addr(token_a, token_b) -> str:
    token0, token1 = sort_addrs(token_a,token_b)
    token0 = Web3.to_bytes(hexstr=token0)
    token1 = Web3.to_bytes(hexstr=token1)
    head = bytes.fromhex('ff') + FactoryAddrV2 
    with_addrs = head + Web3.keccak(encode_packed(['address', 'address'], [token0,token1]))

    with_initcode = Web3.keccak(with_addrs + PoolInitCodeV2)
    i = with_initcode[12:]
    return Web3.to_checksum_address(i)

def get_v3_pool_addr(token_a, token_b,fee) -> str:
    token0, token1 = sort_addrs(token_a,token_b)
    token0 = Web3.to_bytes(hexstr=token0)
    token1 = Web3.to_bytes(hexstr=token1)
    head = bytes.fromhex('ff') + FactoryAddrV3 
    with_addrs = head + Web3.keccak(encode(['address', 'address', 'uint24'], [token0,token1,fee]))

    with_initcode = Web3.keccak(with_addrs + PoolInitCodeV3)
    i = with_initcode[12:]
    return Web3.to_checksum_address(i)

def random_account():
    key = "0x" + secrets.token_hex(32)
    return Account.from_key(key)

if __name__ == '__main__':
    print(get_v2_pool_addr('0xFF970A61A04b1cA14834A43f5dE4533eBDDB5CC8','0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2'))
    print(get_v3_pool_addr('0xFF970A61A04b1cA14834A43f5dE4533eBDDB5CC8','0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2',3000))





import json
import logging
import os
import traceback
from typing import Any, Union, Optional, cast

from eth_account import Account, messages
from eth_account.signers.local import LocalAccount
from eth_typing import URI
from web3 import HTTPProvider
from web3._utils.request import make_post_request
from web3.types import RPCEndpoint, RPCResponse
from web3 import Web3


class BloxProvider(HTTPProvider):
    def __init__(
        self,
        endpoint_uri="https://mev.api.blxrbdn.com",
        auth="",
        request_kwargs=None,
        session=None,
    ) -> None:
        if not auth:
            raise Exception("require blox auth header")
        self.auth = auth
        super().__init__(endpoint_uri, request_kwargs, session)

    def make_request(self, method: RPCEndpoint, params: Any) -> RPCResponse:
        params = params[0]
        self.logger.debug(
            "Making request HTTP. URI: %s, Method: %s", self.endpoint_uri, method
        )

        params = {
                "transaction": [i[2:] for i in params["txs"]],
                "block_number": params["blockNumber"],
                "mev_builders": {
                    "bloxroute": "",
                    "all": "",
                },
            }
        rpc_dict = {
            "jsonrpc": "2.0",
            "method": "blxr_submit_bundle",
            "params": params or [],
            "id": str(next(self.request_counter)),
        }

        try:
            raw_response = make_post_request(
                self.endpoint_uri,
                None,
                json=rpc_dict,
                headers={"Authorization": self.auth},
            ).decode()
        except Exception as e:
            traceback.print_exc()
            raw_response = str(e)
        self.logger.debug(
            "Getting response HTTP. URI: %s, " "Method: %s, Response: %s",
            self.endpoint_uri,
            method,
            raw_response,
        )
        return raw_response



configs = {
    "local": {
        "node": {
            "rpc": "http://localhost:8545",
            "ws": "ws://localhost:8545",
        },
    },
    "ethereum": {
        "node": {
            "rpc": "https://eth-hk1.csnodes.com/v1/973eeba6738a7d8c3bd54f91adcbea89",
            "ws": "wss://eth-hk1.csnodes.com/ws/v1/973eeba6738a7d8c3bd54f91adcbea89",
        },
    },
    "bsc": {
        "node": {
            "rpc": "https://bsc-dataseed1.binance.org",
            "ws": "",
        },
        "is_poa": True,
    },
    "arb": {
        "node": {
            # "rpc": "https://arb-mainnet.g.alchemy.com/v2/7FWmkZwir8-miybmueaw9Nd-hHu0rOO3",
            "rpc": "https://arb1.arbitrum.io/rpc",
            "ws": "wss://arb-mainnet.g.alchemy.com/v2/7FWmkZwir8-miybmueaw9Nd-hHu0rOO3",  # 官方不支持ws
        },
    },
    "zkSync": {
        "node": {
            # "rpc": "https://mainnet.era.zksync.io",
            "rpc": "https://mainnet.era.zksync.io",
            "ws": "wss://mainnet.era.zksync.io/ws",
        },
    },
    "zkFair": {
        "node": {
            # "rpc": "https://mainnet.era.zksync.io",
            "rpc": "https://rpc.zkfair.io",
            "ws": "",
        },
    },
    "avax": {
        "node": {
            "rpc": "https://avalanche-mainnet.core.chainstack.com/ext/bc/C/rpc/021e382ae670cdb8a3b413c463d57763",
            "ws": "wss://avalanche-mainnet.core.chainstack.com/ws/ext/bc/C/ws/021e382ae670cdb8a3b413c463d57763",
        }
    },
    "manta": {
        "node": {
            "rpc": "https://pacific-rpc.manta.network/http",
            "ws": "wss://pacific-rpc.manta.network/ws",
        }
    },
}


uni_config = {
    "v2": {
        "factory": "0x5c69bee701ef814a2b6a3edd4b1652cb9cc5aa6f",
        "init_code": "0x96e8ac4277198ff8b6f785478aa9a39f403cb768dd02cbee326c3e7da348845f",
    },
    "v3": {
        "factory": "0x1F98431c8aD98523631AE4a59f267346ea31F984",
        "init_code": "0xe34f199b19b2b4f47f68442619d555527d244f78a3297ea89325f843f87b8b54",
    },
}



import rlp
import time
from functools import reduce
from typing import Any, Dict, List, Optional, Callable, Union

from eth_account import Account
from eth_account._utils.legacy_transactions import (
    Transaction,
    encode_transaction,
    serializable_unsigned_transaction_from_dict,
)
from eth_account._utils.typed_transactions import (
    AccessListTransaction,
    DynamicFeeTransaction,
)
from eth_typing import HexStr
from hexbytes import HexBytes
from toolz import dissoc
from web3 import Web3
from web3.exceptions import TransactionNotFound
from web3.method import Method
from web3.module import Module
from web3.types import RPCEndpoint, Nonce, TxParams

from .types import (
    FlashbotsOpts,
    FlashbotsBundleRawTx,
    FlashbotsBundleTx,
    FlashbotsBundleDictTx,
    SignedTxAndHash,
    TxReceipt,
)


SECONDS_PER_BLOCK = 12


class FlashbotsRPC:
    eth_sendBundle = RPCEndpoint("eth_sendBundle")


class FlashbotsBundleResponse:
    w3: Web3
    bundle: List[SignedTxAndHash]
    target_block_number: int

    def __init__(self, w3: Web3, txs: List[HexBytes], target_block_number: int):
        self.w3 = w3

        def parse_tx(tx):
            return {
                "signed_transaction": tx,
                "hash": self.w3.keccak(tx),
            }

        self.bundle = list(map(parse_tx, txs))
        self.target_block_number = target_block_number

    def wait(self) -> None:
        """Waits until the target block has been reached"""
        while self.w3.eth.block_number < self.target_block_number:
            time.sleep(1)

    def receipts(self) -> List[TxReceipt]:
        """Returns all the transaction receipts from the submitted bundle"""
        self.wait()
        return list(
            map(lambda tx: self.w3.eth.get_transaction_receipt(tx["hash"]), self.bundle)
        )

    def bundle_hash(self) -> str:
        """Calculates bundle hash"""
        concat_hashes = reduce(
            lambda a, b: a + b,
            map(lambda tx: tx["hash"], self.bundle),
        )
        return self.w3.keccak(concat_hashes)



class Flashbots(Module):
    signed_txs: List[HexBytes]
    response: FlashbotsBundleResponse

    def sign_bundle(
        self,
        bundled_transactions: List[
            Union[FlashbotsBundleTx, FlashbotsBundleRawTx, FlashbotsBundleDictTx]
        ],
    ) -> List[HexBytes]:
        """Given a bundle of signed and unsigned transactions, it signs them all"""
        nonces: Dict[HexStr, Nonce] = {}
        signed_transactions: List[HexBytes] = []

        for tx in bundled_transactions:
            if "signed_transaction" in tx:  # FlashbotsBundleRawTx
                tx_params = _parse_signed_tx(tx["signed_transaction"])
                nonces[tx_params["from"]] = tx_params["nonce"] + 1
                signed_transactions.append(tx["signed_transaction"])

            elif "signer" in tx:  # FlashbotsBundleTx
                signer, tx = tx["signer"], tx["transaction"]
                tx["from"] = signer.address

                if tx.get("nonce") is None:
                    tx["nonce"] = nonces.get(
                        signer.address,
                        self.w3.eth.get_transaction_count(signer.address),
                    )
                nonces[signer.address] = tx["nonce"] + 1

                if "gas" not in tx:
                    tx["gas"] = self.w3.eth.estimateGas(tx)

                signed_tx = signer.sign_transaction(tx)
                signed_transactions.append(signed_tx.rawTransaction)

            elif all(key in tx for key in ["v", "r", "s"]):  # FlashbotsBundleDictTx
                v, r, s = (
                    tx["v"],
                    int(tx["r"].hex(), base=16),
                    int(tx["s"].hex(), base=16),
                )

                tx_dict = {
                    "nonce": tx["nonce"],
                    "data": HexBytes(tx["input"]),
                    "value": tx["value"],
                    "gas": tx["gas"],
                }

                if "maxFeePerGas" in tx or "maxPriorityFeePerGas" in tx:
                    assert "maxFeePerGas" in tx and "maxPriorityFeePerGas" in tx
                    tx_dict["maxFeePerGas"], tx_dict["maxPriorityFeePerGas"] = (
                        tx["maxFeePerGas"],
                        tx["maxPriorityFeePerGas"],
                    )
                else:
                    assert "gasPrice" in tx
                    tx_dict["gasPrice"] = tx["gasPrice"]

                if tx.get("accessList"):
                    tx_dict["accessList"] = tx["accessList"]

                if tx.get("chainId"):
                    tx_dict["chainId"] = tx["chainId"]

                if tx.get("to"):
                    tx_dict["to"] = HexBytes(tx["to"])

                unsigned_tx = serializable_unsigned_transaction_from_dict(tx_dict)
                raw = encode_transaction(unsigned_tx, vrs=(v, r, s))
                assert self.w3.keccak(raw) == tx["hash"]
                signed_transactions.append(raw)

        return signed_transactions

    def to_hex(self, signed_transaction: bytes) -> str:
        tx_hex = signed_transaction.hex()
        if tx_hex[0:2] != "0x":
            tx_hex = f"0x{tx_hex}"
        return tx_hex

    def send_raw_bundle_munger(
        self,
        signed_bundled_transactions: List[HexBytes],
        target_block_number: int,
        opts: Optional[FlashbotsOpts] = None,
    ) -> List[Any]:
        """Given a raw signed bundle, it packages it up with the block number and the timestamps"""

        if opts is None:
            opts = {}

        # convert to hex
        return [
            {
                "txs": list(map(lambda x: self.to_hex(x), signed_bundled_transactions)),
                "blockNumber": hex(target_block_number),
                "minTimestamp": opts["minTimestamp"] if "minTimestamp" in opts else 0,
                "maxTimestamp": opts["maxTimestamp"] if "maxTimestamp" in opts else 0,
                "revertingTxHashes": opts["revertingTxHashes"]
                if "revertingTxHashes" in opts
                else [],
                "replacementUuid": opts["replacementUuid"]
                if "replacementUuid" in opts
                else None,
            }
        ]

    sendRawBundle: Method[Callable[[Any], Any]] = Method(
        FlashbotsRPC.eth_sendBundle, mungers=[send_raw_bundle_munger]
    )
    send_raw_bundle = sendRawBundle

    def send_bundle_munger(
        self,
        bundled_transactions: List[Union[FlashbotsBundleTx, FlashbotsBundleRawTx]],
        target_block_number: int,
        opts: Optional[FlashbotsOpts] = None,
    ) -> List[Any]:
        signed_txs = self.sign_bundle(bundled_transactions)
        self.response = FlashbotsBundleResponse(
            self.w3, signed_txs, target_block_number
        )
        return self.send_raw_bundle_munger(signed_txs, target_block_number, opts)

    def raw_bundle_formatter(self, resp) -> Any:
        return lambda _: resp.response

    sendBundle: Method[Callable[[Any], Any]] = Method(
        FlashbotsRPC.eth_sendBundle,
        mungers=[send_bundle_munger],
        result_formatters=raw_bundle_formatter,
    )
    send_bundle = sendBundle


    def extrapolate_timestamp(self, block_tag: int, latest_block_number: int):
        block_delta = block_tag - latest_block_number
        if block_delta < 0:
            raise Exception("block extrapolation negative")
        return self.w3.eth.get_block(latest_block_number)["timestamp"] + (
            block_delta * SECONDS_PER_BLOCK
        )



def _parse_signed_tx(signed_tx: HexBytes) -> TxParams:
    # decode tx params based on its type
    tx_type = signed_tx[0]
    if tx_type > int("0x7f", 16):
        # legacy and EIP-155 transactions
        decoded_tx = rlp.decode(signed_tx, Transaction).as_dict()
    else:
        # typed transactions (EIP-2718)
        if tx_type == 1:
            # EIP-2930
            sedes = AccessListTransaction._signed_transaction_serializer
        elif tx_type == 2:
            # EIP-1559
            sedes = DynamicFeeTransaction._signed_transaction_serializer
        else:
            raise ValueError(f"Unknown transaction type: {tx_type}.")
        decoded_tx = rlp.decode(signed_tx[1:], sedes).as_dict()

    # recover sender address and remove signature fields
    decoded_tx["from"] = Account.recover_transaction(signed_tx)
    decoded_tx = dissoc(decoded_tx, "v", "r", "s")
    return decoded_tx



from typing import Callable
from web3.types import *
from web3.middleware.signing import *
from eth_utils.toolz import (
    compose,
)

import secrets
import hashlib
import hmac
from Crypto.Cipher import AES
from hashlib import sha512
from binascii import unhexlify
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
import requests
import json
from web3 import Web3


def signer_middleware(w3: Web3, url: str, pubkey: str) -> Middleware:
    signer = RemoteSigner(w3.eth.chain_id, url, pubkey)

    def sign_and_send_raw_middleware(
        make_request: Callable[[RPCEndpoint, Any], Any], w3: "Web3"
    ) -> Callable[[RPCEndpoint, Any], RPCResponse]:
        format_and_fill_tx = compose(
            format_transaction, fill_transaction_defaults(w3), fill_nonce(w3)
        )

        def middleware(method: RPCEndpoint, params: Any) -> RPCResponse:
            if method != "eth_sendTransaction":
                return make_request(method, params)
            else:
                transaction = format_and_fill_tx(params[0])
            if "from" not in transaction:
                return make_request(method, params)
            elif not transaction.get("from"):
                return make_request(method, params)

            fm = transaction['from']
            if 'maxFeePerDataGas' in transaction:
                transaction['type'] = '0x3'
            elif 'maxFeePerGas' in transaction and 'maxPriorityFeePerGas' in transaction:
                transaction['type'] = '0x2'
            elif 'acccessList' in transaction:
                transaction['type'] = '0x1'
            else:
                transaction['type'] = '0x0'
            
            result = signer.sign_transaction(fm, transaction)
            return make_request(RPCEndpoint("eth_sendRawTransaction"), [result['hex']])

        return middleware

    return sign_and_send_raw_middleware


AES_BLOCK_SIZE = 16


class EncryptOption:
    def __init__(self, iv, ephemPublicKey, ciphertext, mac):
        self.iv = iv
        self.ephemPublicKey = ephemPublicKey
        self.ciphertext = ciphertext
        self.mac = mac

    def __str__(self):
        return f"Iv: {self.iv}, \nEphemPublicKey: {self.ephemPublicKey}, \nCiphertext: {self.ciphertext}, \nMac: {self.mac}"

    def stringify(self):
        compressed_key = compress(self.ephemPublicKey)

        data = bytearray()
        data.extend(bytes.fromhex(self.iv))
        data.extend(compressed_key)
        data.extend(bytes.fromhex(self.mac))
        data.extend(bytes.fromhex(self.ciphertext))

        return data.hex()


def compress(pubkey):
    pub_key = bytes.fromhex(pubkey)

    if pub_key[0] != 0x04 or len(pub_key) != 65:
        raise ValueError("Invalid public key")

    result = bytearray(33)
    if pub_key[64] % 2 == 1:
        result[0] = 0x03
    else:
        result[0] = 0x02

    result[1:] = pub_key[1:33]
    return result


def encrypt_by_key(public_key: str, message: bytes):
    public_key_bytes = Web3.to_bytes(hexstr=public_key)
    # Load public key
    public_key_numbers = ec.EllipticCurvePublicNumbers(
        curve=ec.SECP256K1(),
        x=int.from_bytes(public_key_bytes[1:33], byteorder="big"),
        y=int.from_bytes(public_key_bytes[33:], byteorder="big"),
    )

    public_key = public_key_numbers.public_key(default_backend())

    # Generate ephemeral key
    ephemeral_private_key = ec.generate_private_key(ec.SECP256K1(), default_backend())
    ephemeral_public_key = ephemeral_private_key.public_key()

    # ECDH
    shared_secret = ephemeral_private_key.exchange(ec.ECDH(), public_key)
    derived_key = hashlib.sha512(shared_secret).digest()
    key_e = derived_key[:32]
    key_m = derived_key[32:]

    # AES Encrypt
    padder = padding.PKCS7(128).padder()
    padded_data = padder.update(message) + padder.finalize()

    iv = secrets.token_bytes(16)

    cipher = Cipher(algorithms.AES(key_e), modes.CBC(iv), default_backend())
    encryptor = cipher.encryptor()
    ciphertext = encryptor.update(padded_data) + encryptor.finalize()

    # HMAC
    data_to_mac = (
        iv
        + ephemeral_public_key.public_bytes(
            encoding=serialization.Encoding.X962,
            format=serialization.PublicFormat.UncompressedPoint,
        )
        + ciphertext
    )
    hmac_calculator = hmac.new(key_m, data_to_mac, hashlib.sha256)
    mac = hmac_calculator.digest()

    return EncryptOption(
        iv=iv.hex(),
        ephemPublicKey=ephemeral_public_key.public_bytes(
            encoding=serialization.Encoding.X962,
            format=serialization.PublicFormat.UncompressedPoint,
        ).hex(),
        ciphertext=ciphertext.hex(),
        mac=mac.hex(),
    )


def decrypt_by_key(opt: EncryptOption, prv_key_str: str):
    if opt is None:
        raise ValueError("EncryptOption data error")

    if not prv_key_str.startswith("0x"):
        prv_key_str = "0x" + prv_key_str

    byte_key = bytes.fromhex(prv_key_str[2:])
    int_key = int.from_bytes(byte_key, byteorder="big")

    # 使用 coincurve 创建私钥和公钥对象
    curve = ec.SECP256K1()
    private_key = ec.derive_private_key(int_key, curve, default_backend())

    # Load the public key
    ephem_public_key_bytes = unhexlify(opt.ephemPublicKey)
    public_key = ec.EllipticCurvePublicKey.from_encoded_point(
        ec.SECP256K1(), ephem_public_key_bytes
    )

    # 计算ECDH共享密钥
    ecdh_key = private_key.exchange(ec.ECDH(), public_key)

    derived_key = sha512(ecdh_key).digest()
    key_e = derived_key[:32]
    key_m = derived_key[32:]

    # 先将opt.iv作为第一个元素加入
    data_to_mac = [opt.iv]
    # 将opt.ephemPublicKey和opt.ciphertext拼接
    combined = opt.ephemPublicKey + opt.ciphertext

    # 再将combined作为第二个元素加入
    data_to_mac.append(combined)
    res = "".join(data_to_mac)

    bytes_data_to_mac = bytes.fromhex(res)
    hm = hmac.new(bytes.fromhex(key_m.hex()), bytes_data_to_mac, hashlib.sha256)
    expected_mac = hm.digest()

    if not hmac.compare_digest(opt.mac, expected_mac.hex()):
        raise ValueError("Invalid MAC")

    iv_bytes = bytes.fromhex(opt.iv)
    cipher = AES.new(key_e, AES.MODE_CBC, iv_bytes)
    plain_text = cipher.decrypt(bytes.fromhex(opt.ciphertext))

    # Assuming you have a removePKCSPadding function implemented in Python
    plain = remove_pkcs_padding(plain_text)
    return plain


def remove_pkcs_padding(src):
    length = len(src)
    pad_length = src[-1]

    if pad_length > AES_BLOCK_SIZE or length < AES_BLOCK_SIZE:
        raise ValueError(f"invalid padding length: {pad_length}")
    return src[:-pad_length]


class RemoteSigner:
    SIGNER_PING = "/ping"
    SIGNER_ADDRESS = "/v1/address"
    SIGNER_SIGN_TRANSACTION = "/v1/sign/transaction"
    SIGNER_SIGN_MESSAGE = "/v1/sign/message"
    SIGNER_SIGN_EIP712 = "/v1/sign/eip712"

    def __init__(self, chainId: int, host, pub_key: str):
        self.chain_id = chainId
        self.host = host
        self.pub_key = pub_key

    def ping(self):
        response = requests.get(self.host + self.SIGNER_PING)
        return response.json()

    def get_address(self, index: int):
        param = dict(
            chain_id=self.chain_id,
            index=index,
        )
        body = self.encrypt_body(param)
        host = self.host + self.SIGNER_ADDRESS
        resp = requests.post(host, data=body)
        if resp.status_code != 200:
            raise Exception(f"failed get address {resp.status_code} {resp.text}")
        return resp.json()["data"]

    def sign_message(self, address, message: str):
        param = dict(
            chain_id=self.chain_id,
            account=address,
            message=message,
        )

        body = self.encrypt_body(param)
        host = self.host + self.SIGNER_SIGN_MESSAGE
        resp = requests.post(host, data=body)
        if resp.status_code != 200:
            raise Exception(f"failed sign message {resp.status_code} {resp.text}")

        return resp.json()["data"]

    def sign_transaction(self, address: str, tx: TxParams):

        for k,v in tx.items():
            if isinstance(v, int):
                tx[k] = Web3.to_hex(v)

        transaction_hash = Web3.keccak(b'')
        tx['hash'] = Web3.to_hex(transaction_hash)

        transaction_str = Web3.to_json(tx)
        param = dict(
            chain_id=self.chain_id,
            account=address,
            transaction=transaction_str,
        )
        print(param)

        body = self.encrypt_body(param)
        host = self.host + self.SIGNER_SIGN_TRANSACTION
        resp = requests.post(host, data=body)
        if resp.status_code != 200:
            raise Exception(f"failed sign transaction {resp.status_code} {resp.text}")
        response_json = resp.json()
        return {
            "tx": response_json["tx"],
            "hex": response_json["tx_hex"],
        }

    def sign_eip712(self, address, eip712_data):
        param = dict(
            chain_id=self.chain_id,
            account=address,
            data=eip712_data,
        )

        body = self.encrypt_body(param)
        host = self.host + self.SIGNER_SIGN_EIP712
        resp = requests.post(host, data=body)
        if resp.status_code != 200:
            raise Exception(f"failed sign 712 {resp.status_code} {resp.text}")
        return resp.json()["signature"]

    def encrypt_body(self, req_data):
        data = json.dumps(req_data).encode()
        encrypted_data = encrypt_by_key(self.pub_key, data)
        ed = encrypted_data.stringify()
        body = {"encryptMsg": "0x" + ed}
        return body



import time
from web3 import Web3
from ether.client import Web3Client
from ether.utils import sort_addrs
from .chains import chains
from eth_abi import encode
from eth_abi.packed import encode_packed
from web3.middleware import construct_sign_and_send_raw_middleware
from . import abis


class Meta(type):
    def __getitem__(self, arg):
        return UniswapV3(Web3Client[arg], chains[arg])


class UniswapV3(metaclass=Meta):
    def __init__(self, web3_client: Web3Client, conf: dict) -> None:
        self.conf = conf
        self.client = web3_client
        self.swap_router = web3_client.eth.contract(
            conf["router"], abi=abis.swap_router
        )

    def with_account(self, private_key: str):
        self.acc = self.client.eth.account.from_key(private_key)
        self.client.middleware_onion.add(construct_sign_and_send_raw_middleware(self.acc))
        self.client.eth.default_account = self.acc.address
        return self

    def get_pool_addr(self, token_a: str, token_b: str, fee: int):
        token0, token1 = sort_addrs(token_a, token_b)

        token0 = Web3.to_bytes(hexstr=token0)
        token1 = Web3.to_bytes(hexstr=token1)
        head = bytes.fromhex("ff") + Web3.to_bytes(hexstr=self.conf["factory"])
        with_addrs = head + Web3.keccak(
            encode(["address", "address", "uint24"], [token0, token1, fee])
        )

        with_initcode = Web3.keccak(
            with_addrs + Web3.to_bytes(hexstr=self.conf["init_code"])
        )
        i = with_initcode[12:]
        return Web3.to_checksum_address(i)

    def build_exactInputSingle(
        self,
        tokenin,
        tokenout,
        fee,
        receipient: str,
        amount_in,
        min_amount_out,
        deadline=None,
        sqrtPriceLimitX96=0,
    ):
        """
        根据参数组装出交易的input
        """
        if not deadline:
            deadline = int(time.time() + 60)
        return self.swap_router.functions.exactInputSingle(
            (
                tokenin,
                tokenout,
                fee,
                receipient,
                deadline,
                amount_in,
                min_amount_out,
                sqrtPriceLimitX96,
            )
        )



import json
from eth_account import Account
from web3 import Web3
from eth_account.signers.local import LocalAccount


class Wallet:
    def __init__(self, mnemonic) -> None:
        self.mnemonic = mnemonic

    @classmethod
    def load_keystore(cls, password, f="keystore.json"):
        return cls.decrypt_keystore(json.load(open(f)), password)

    @classmethod
    def decrypt_keystore(cls, data, password):
        mn = Account.decrypt(data, password).decode()
        return Wallet(mn)

    def __getitem__(self, index) -> list[LocalAccount]:
        if isinstance(index, slice):
            return [
                Account.from_mnemonic(self.mnemonic, account_path=f"m/44'/60'/0'/0/{i}")
                for i in range(index.start, index.stop)
            ]
        else:
            return Account.from_mnemonic(
                self.mnemonic, account_path=f"m/44'/60'/0'/0/{index}"
            )



providers = [
    "https://relay.flashbots.net",
    "https://builder0x69.io",
    "https://eth-builder.com",
    "https://rpc.beaverbuild.org",
    "https://rsync-builder.xyz",
    "https://api.edennetwork.io/v1/bundle",
    "https://rpc.titanbuilder.xyz",
    "https://rpc.penguinbuild.org",
    "https://rpc.nfactorial.xyz",
    "https://builder.gmbit.co/rpc",
    "https://rpc.jetbldr.xyz",
    "https://api.securerpc.com/v1",
    "https://BuildAI.net",
    "https://rpc.lokibuilder.xyz",
    "https://rpc.lightspeedbuilder.info",
    "https://rpc.payload.de",
    "https://rpc.f1b.io",
]


from eth_account import Account
from web3 import Web3
from ether.chains import configs
from ether.middlewares.remote_signer import signer_middleware
from ether.utils import random_account
from web3.middleware import construct_sign_and_send_raw_middleware, geth_poa_middleware


from ether.ws import WsClient
from ether.flashbots import flashbot
from ether.mev import providers


class Meta(type):
    def __getitem__(self, arg):
        return Web3Client(chain_config=configs[arg])


class Web3Client(Web3, metaclass=Meta):
    def __init__(
        self, chain_config=configs["ethereum"], pk=None, request_kwargs=None
    ) -> None:
        super().__init__(
            Web3.HTTPProvider(
                chain_config["node"]["rpc"], request_kwargs=request_kwargs
            )
        )
        # bsc
        if chain_config.get("is_poa"):
            self.middleware_onion.inject(geth_poa_middleware, layer=0)
        self.config = chain_config
        self.w3 = self
        if pk:
            self.with_account(pk)

    def with_account(self, private_key: str):
        self.acc = self.eth.account.from_key(private_key)
        self.middleware_onion.add(construct_sign_and_send_raw_middleware(self.acc))
        self.eth.default_account = self.acc.address
        return self

    def with_signer(self, url, pubkey):
        self.middleware_onion.add(signer_middleware(self, url, pubkey))
        return self

    def with_mev(self, blox_token: str):
        rk = random_account()
        signer = Account.from_key(rk.key)
        flashbot(self, signer, blox_token, providers)
        return self

    def nonce(self):
        return self.eth.get_transaction_count(self.acc.address)

    def subscribe_txs(self):
        return WsClient(self.config["node"]["ws"], ["newPendingTransactions", True])

    def subscribe_blocks(self):
        return WsClient(self.config["node"]["ws"], ["newHeads"])

    def subscribe_logs(self, address=None, topics=[]):
        return WsClient(
            self.config["node"]["ws"], ["logs", {"address": address, "topics": topics}]
        )


if __name__ == "__main__":
    print(Web3Client["ethereum"])



from typing import Union, Optional

from eth_account.signers.local import LocalAccount
from eth_typing import URI
from web3 import Web3
from web3.middleware import geth_poa_middleware
from web3._utils.module import attach_modules

from .blox import BloxProvider

from .flashbots import Flashbots
from .middleware import construct_flashbots_middleware
from .provider import FlashbotProvider

DEFAULT_FLASHBOTS_RELAY = "https://relay.flashbots.net"


def flashbot(
    w3: Web3,
    signature_account: LocalAccount,
    blox_token: str,
    endpoint_uris: list[str],
):
    """
    Injects the flashbots module and middleware to w3.
    """

    flashbots_providers = [FlashbotProvider(signature_account, i) for i in endpoint_uris]
    # flashbots_providers = []
    # bloXroute
    flashbots_providers.append(BloxProvider(auth=blox_token))

    flash_middleware = construct_flashbots_middleware(flashbots_providers)
    w3.middleware_onion.add(flash_middleware)

    # attach modules to add the new namespace commands
    attach_modules(w3, {"flashbots": (Flashbots,)})



import json

erc20 = json.loads(
    """[
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "name_",
        "type": "string"
      },
      {
        "internalType": "string",
        "name": "symbol_",
        "type": "string"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "owner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "spender",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "value",
        "type": "uint256"
      }
    ],
    "name": "Approval",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "value",
        "type": "uint256"
      }
    ],
    "name": "Transfer",
    "type": "event"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "owner",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "spender",
        "type": "address"
      }
    ],
    "name": "allowance",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "spender",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "approve",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "balanceOf",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "decimals",
    "outputs": [
      {
        "internalType": "uint8",
        "name": "",
        "type": "uint8"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "spender",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "subtractedValue",
        "type": "uint256"
      }
    ],
    "name": "decreaseAllowance",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "spender",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "addedValue",
        "type": "uint256"
      }
    ],
    "name": "increaseAllowance",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "name",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "symbol",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "totalSupply",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "transfer",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "transferFrom",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  }
]"""
)

erc721 = json.loads('''
[
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "name_",
        "type": "string"
      },
      {
        "internalType": "string",
        "name": "symbol_",
        "type": "string"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "owner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "approved",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "Approval",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "owner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "operator",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "approved",
        "type": "bool"
      }
    ],
    "name": "ApprovalForAll",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "Transfer",
    "type": "event"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "approve",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "owner",
        "type": "address"
      }
    ],
    "name": "balanceOf",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "getApproved",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "owner",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "operator",
        "type": "address"
      }
    ],
    "name": "isApprovedForAll",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "name",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "ownerOf",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "safeTransferFrom",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      },
      {
        "internalType": "bytes",
        "name": "_data",
        "type": "bytes"
      }
    ],
    "name": "safeTransferFrom",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "operator",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "approved",
        "type": "bool"
      }
    ],
    "name": "setApprovalForAll",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes4",
        "name": "interfaceId",
        "type": "bytes4"
      }
    ],
    "name": "supportsInterface",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "symbol",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "tokenURI",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "tokenId",
        "type": "uint256"
      }
    ],
    "name": "transferFrom",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  }
]
''')

erc1155 = json.loads("""
[
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "uri_",
        "type": "string"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "operator",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "approved",
        "type": "bool"
      }
    ],
    "name": "ApprovalForAll",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "operator",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256[]",
        "name": "ids",
        "type": "uint256[]"
      },
      {
        "indexed": false,
        "internalType": "uint256[]",
        "name": "values",
        "type": "uint256[]"
      }
    ],
    "name": "TransferBatch",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "operator",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "value",
        "type": "uint256"
      }
    ],
    "name": "TransferSingle",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "string",
        "name": "value",
        "type": "string"
      },
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      }
    ],
    "name": "URI",
    "type": "event"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      }
    ],
    "name": "balanceOf",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address[]",
        "name": "accounts",
        "type": "address[]"
      },
      {
        "internalType": "uint256[]",
        "name": "ids",
        "type": "uint256[]"
      }
    ],
    "name": "balanceOfBatch",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "operator",
        "type": "address"
      }
    ],
    "name": "isApprovedForAll",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256[]",
        "name": "ids",
        "type": "uint256[]"
      },
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      },
      {
        "internalType": "bytes",
        "name": "data",
        "type": "bytes"
      }
    ],
    "name": "safeBatchTransferFrom",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      },
      {
        "internalType": "bytes",
        "name": "data",
        "type": "bytes"
      }
    ],
    "name": "safeTransferFrom",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "operator",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "approved",
        "type": "bool"
      }
    ],
    "name": "setApprovalForAll",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes4",
        "name": "interfaceId",
        "type": "bytes4"
      }
    ],
    "name": "supportsInterface",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "name": "uri",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  }
]
""")


weth = json.loads("""[
  {
    "constant": true,
    "inputs": [],
    "name": "name",
    "outputs": [
      {
        "name": "",
        "type": "string"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "guy",
        "type": "address"
      },
      {
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "approve",
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "totalSupply",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "src",
        "type": "address"
      },
      {
        "name": "dst",
        "type": "address"
      },
      {
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "transferFrom",
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "withdraw",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "decimals",
    "outputs": [
      {
        "name": "",
        "type": "uint8"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "name": "balanceOf",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "symbol",
    "outputs": [
      {
        "name": "",
        "type": "string"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "dst",
        "type": "address"
      },
      {
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "transfer",
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [],
    "name": "deposit",
    "outputs": [],
    "payable": true,
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "",
        "type": "address"
      },
      {
        "name": "",
        "type": "address"
      }
    ],
    "name": "allowance",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "payable": true,
    "stateMutability": "payable",
    "type": "fallback"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "name": "src",
        "type": "address"
      },
      {
        "indexed": true,
        "name": "guy",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "Approval",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "name": "src",
        "type": "address"
      },
      {
        "indexed": true,
        "name": "dst",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "Transfer",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "name": "dst",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "Deposit",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "name": "src",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "wad",
        "type": "uint256"
      }
    ],
    "name": "Withdrawal",
    "type": "event"
  }
]""")


from setuptools import setup, find_packages

setup(
    name="liechains",
    version="0.2.2",
    author="leeliewhite",
    author_email="leeliewhite@gmail.com",
    description="toolkit for blockchains",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "web3",
        "websocket-client"
    ],
    python_requires=">=3.9",
)
